﻿/****************************************************************************/
/* Copyright (C) 2009-2011, Heartland.Data.co.  All Rights Reserved.		*/
/*																			*/
/* Title	: DT10 Driver	[All]											*/
/* Author	: HLDC															*/
/****************************************************************************/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#include "dt10_tpdrv.h"

#define DEVNAME "/dev/dt10drvgpio"	/* Device file */

/* ========== Parameter ========== */
#define		DT_UINT32				unsigned int		/* Test Point argument type  */
#define		DT_VARIABLE_BIT			0x02				/* Memory Dump MASK */
#define		DT_EVTTRG_BIT			0x08				/* EventTrigger */
#define		DT_VARIABLE_FAST_BIT	0x01				/* Memory Dump MASK(Fast Mode) */
#define		DT_EVTTRG_FAST_BIT		0x02				/* EventTrigger(Fast Mode) */
/* ============================================ */


static int fd = -1;

/*===============================================================================
	Func	: _TP_BusOutOpen
	Desc	: Open device driver for Test Point.
===============================================================================*/
void	_TP_BusOutOpen(void)
{
	fd = open(DEVNAME, O_RDWR);
	if( fd < 0 )
	{
		perror("open");
	}
}

/*===============================================================================
	Func	: _TP_BusOutClose
	Desc	: Close device driver for Test Point.
===============================================================================*/
void	_TP_BusOutClose(void)
{
	if( fd >= 0 )
	{
		close(fd);
		fd = -1;
	}
}

/*===============================================================================
	Func	: _TP_BusOut
	Desc	: Called by Test Point
===============================================================================*/
void _TP_BusOut( DT_UINT32 addr, DT_UINT32 dat )
{
	int	ret;
	dt10_tp	tp;	

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr		= addr;
	tp.data		= dat;
	tp.size		= 0;
	tp.fast_mode = 0;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*===============================================================================
	Func	: _TP_MemoryOutput
	Desc	: Called by Variable Test Point
===============================================================================*/
void _TP_MemoryOutput( DT_UINT32 addr, DT_UINT32 dat, void *value, DT_UINT32 size)
{
	dt10_tp	tp;	
	int	i;
	int	ret;
	unsigned char *p;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr		= addr | DT_VARIABLE_BIT ;			/* Memory Dump Start */ 
	tp.data		= dat ;
	tp.fast_mode = 0;

	if (size >= MAX_TP_DATA) size = MAX_TP_DATA;
	tp.p[0] = size;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i++) {
			tp.p[1+i] = p[i];		/* Memory Dump */
		}
	}
	tp.size = size + 1;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*===============================================================================
	Func	: _TP_EventTrigger
	Desc	: Called by Event Trigger Test Point
===============================================================================*/
void _TP_EventTrigger( DT_UINT32 addr, DT_UINT32 dat)
{
	dt10_tp	tp;	
	int	ret;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr		= addr | DT_EVTTRG_BIT ;
	tp.data		= ((unsigned short)(dat & 0x0FFF)) << 4 ;
	tp.size		= 0;
	tp.fast_mode = 0;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*===============================================================================
	Func	: _TP_BusOutFast
	Desc	: Called by Fast Test Point
===============================================================================*/
void _TP_BusOutFast( DT_UINT32 step, DT_UINT32 bit )
{
	int	ret;
	dt10_tp	tp;	

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr		= step << 2;
	tp.data		= bit + 2;
	tp.size		= 0;
	tp.fast_mode = 1;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*===============================================================================
	Func	: _TP_MemoryOutputFast
	Desc	: Called by Fast Variable Test Point
===============================================================================*/
void _TP_MemoryOutputFast( DT_UINT32 step, DT_UINT32 bit, void *value, DT_UINT32 size)
{
	dt10_tp	tp;	
	int	i;
	int	ret;
	unsigned char *p;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr		= (step << 2) + DT_VARIABLE_FAST_BIT;		/* Memory Dump Start */ 
	tp.data		= bit + 2;
	tp.fast_mode = 1;

	if (size >= MAX_TP_DATA) size = MAX_TP_DATA;
	tp.p[0] = size;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i++) {
			tp.p[1+i] = p[i];		/* Memory Dump */
		}
	}
	tp.size = size + 1;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*===============================================================================
	Func	: _TP_EventTriggerFast
	Desc	: Called by Fast Event Trigger Test Point
===============================================================================*/
void _TP_EventTriggerFast(DT_UINT32 dat)
{
	dt10_tp	tp;	
	int	ret;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr		= DT_EVTTRG_FAST_BIT;
	tp.data		= dat;
	tp.size		= 0;
	tp.fast_mode = 2;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}
