﻿/****************************************************************************/
/* Copyright (C) 2009-2011, Heartland.Data.co.  All Rights Reserved.		*/
/*																			*/
/* Title	: DT10 Driver	[All]											*/
/* Author	: HLDC															*/
/****************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/string.h>
#include <linux/miscdevice.h>
#include <asm/io.h>
#include <linux/ep93xx_gpio.h>

#include "dt10_tpdrv.h"

#define LICENSE "GPL v2"

#define	CS_BIT		0x00000001	/* CS port number */
#define	CLK_BIT		0x00000002	/* CLK port number */
#define	DATA_BIT	0x000000f0	/* DATA port number */

#define	CLK_CS_HI	{			\
	int bit;						\
	bit = inl(GPIO_PBDR);		\
	bit |= CLK_BIT | CS_BIT; 	\
	outl(bit, GPIO_PBDR);		\
}

#define	CLK_HI	{				\
	int bit;						\
	bit = inl(GPIO_PBDR);		\
	bit |= CLK_BIT; 				\
	outl(bit, GPIO_PBDR);		\
}
#define	CLK_LOW	{			\
	int bit;						\
	bit = inl(GPIO_PBDR);		\
	bit &= 窶ｾCLK_BIT; 			\
	outl(bit, GPIO_PBDR);		\
}

#define	CS_HI	{				\
	int bit;						\
	bit = inl(GPIO_PBDR);		\
	bit |= CS_BIT; 				\
	outl(bit, GPIO_PBDR);		\
}
#define	CS_LOW	{				\
	int bit;						\
	bit = inl(GPIO_PBDR);		\
	bit &= 窶ｾCS_BIT; 				\
	outl(bit, GPIO_PBDR);		\
}

#define DATA_OUT(x) {			\
	int bit;						\
	bit = inl(GPIO_PADR);		\
	bit &= 窶ｾDATA_BIT; 			\
	bit |= (x << 4) & DATA_BIT;		\
	outl(bit, GPIO_PADR);		\
}

/*===============================================================================
	Func	: dt10drvgpio_open
	Desc	: This function is executed when opening the device driver.
===============================================================================*/
static int dt10drvgpio_open(struct inode *inode, struct file *filp)
{
	unsigned int bit ;

	/* Port A */
	bit = inl(GPIO_PADDR);
	bit |= DATA_BIT;
	outl(bit, GPIO_PADDR);

	/* Port B */
	bit = inl(GPIO_PBDDR);
	bit |= CLK_BIT | CS_BIT;
	outl(bit, GPIO_PBDDR);

	CLK_CS_HI;

	printk("dt10drvgpio: open\n");
	return 0;
}

/*===============================================================================
	Func	: gpio_busout
	Desc	: Called by Test Point.
===============================================================================*/
static void gpio_busout( unsigned int addr, unsigned int data )
{
	CS_LOW;

	DATA_OUT(data >> 12);		// bit15-12	
	CLK_LOW;

	DATA_OUT(data >> 8);		// bit11-8	
	CLK_HI;

	DATA_OUT(data >> 4);		// bit7-4	
	CLK_LOW;

	DATA_OUT(data);				// bit3-0	
	CLK_HI;

	if( addr >= 0x00010000 )
	{
		DATA_OUT(addr >> 20);		// bit23-20	
		CLK_LOW;
		DATA_OUT(addr >> 16);		// bit19-16	
		CLK_HI;
	}

	if( addr >= 0x00000100 )
	{
		DATA_OUT(addr >> 12);		// bit15-12	
		CLK_LOW;
		DATA_OUT(addr >> 8);		// bit11-8	
		CLK_HI;
	}

	DATA_OUT(addr >> 4);			// bit7-4	
	CLK_LOW;

	DATA_OUT(addr);					// bit3-0	
	CLK_HI;

	CS_HI;

}

/*===============================================================================
	Func	: gpio_memoryout
	Desc	: Test Point Output Function.
===============================================================================*/
static void gpio_memoryout(unsigned char data)
{
	DATA_OUT(data >> 4 );			// bit7-4
	CLK_LOW

	DATA_OUT(data);				// bit3-0
	CLK_HI;
}

/*===============================================================================
	Func	: gpio_busoutfast
	Desc	: Test Point Output Function.
===============================================================================*/
static void gpio_busoutfast(unsigned int data, unsigned int bit)
{
	CS_LOW;
	if( bit >= 9 ){
		DATA_OUT(data >> 8);
		CLK_LOW;
		DATA_OUT(data >> 4);
		CLK_HI;
		DATA_OUT(data);
		CLK_LOW;
	}else if( bit < 9 && bit >= 5 ){
		DATA_OUT(data >> 4);
		CLK_LOW;
		DATA_OUT(data);
		CLK_HI;
	}else{
		DATA_OUT(data);
		CLK_LOW;
	}

	CS_HI;
	CLK_HI;
}

/*===============================================================================
	Func	: gpio_eventout
	Desc	: Test Point Output Function.
===============================================================================*/
static void gpio_eventout(unsigned int addr, unsigned int data)
{
	CS_LOW;
	DATA_OUT(addr);
	CLK_LOW;
	CS_HI;
	CLK_HI;
	CS_LOW;
	gpio_memoryout((data));
	gpio_memoryout((data >> 8));
	gpio_memoryout(0x00);
	gpio_memoryout(0x00);
	CS_HI;
}

/*===============================================================================
	Func	: dt10drvgpio_write
	Desc	: This function is executed upon write instruction.
===============================================================================*/
static int dt10drvgpio_write(struct file *filp, char *buff, size_t count, loff_t *pos)
{
	dt10_tp *tp;
	int	i;
	
	tp = (dt10_tp *)buff ;
	switch (tp->fast_mode) {
	case 0:					/* Normal Mode */
		gpio_busout(tp->addr, tp->data);
		break;
	case 1:					/* Fast Mode */
		gpio_busoutfast(tp->addr, tp->data);
		break;
	case 2:					/* Fast Mode Event Trigger */
		gpio_eventout(tp->addr, tp->data);
		break;
	default:
		break;
	}
	if( tp->size != 0 ){
		CS_LOW;
		for (i = 0; i < tp->size; i++) {
			gpio_memoryout(tp->p[i]);
		}
		CS_HI;
	}
	return count ;

}

/**********************************kernel TestPoint API**********************************/
static void gpio_write(dt10_tp tp)
{
	int	i;
	
	switch (tp.fast_mode) {
	case 0:					/* Normal Mode */
		gpio_busout(tp.addr, tp.data);
		break;
	case 1:					/* Fast Mode */
		gpio_busoutfast(tp.addr, tp.data);
		break;
	case 2:					/* Fast Mode Event Trigger */
		gpio_eventout(tp.addr, tp.data);
		break;
	default:
		break;
	}
	if( tp.size != 0 ){
		CS_LOW;
		for (i = 0; i < tp.size; i++) {
			gpio_memoryout(tp.p[i]);
		}
		CS_HI;
	}
}

void _TP_BusOut( unsigned int addr, unsigned int dat )
{
	dt10_tp	tp;

	tp.addr		= addr;
	tp.data		= dat;
	tp.size		= 0;
	tp.fast_mode = 0;
	gpio_write(tp);
}
EXPORT_SYMBOL(_TP_BusOut);

void _TP_MemoryOutput( unsigned int addr, unsigned int dat, void *value, unsigned int size)
{
	dt10_tp	tp;
	int	i;
	unsigned char *p;

	tp.addr		= addr | 0x02;			/* Memory Dump MASK */
	tp.data		= dat;
	tp.fast_mode = 0;

	if (size >= 256) size = 256;
	tp.p[0] = size;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i++) {
			tp.p[1+i] = p[i];		/* Memory Dump */
		}
	}
	tp.size = size + 1;
	gpio_write(tp);
}
EXPORT_SYMBOL(_TP_MemoryOutput);

void _TP_EventTrigger( unsigned int addr, unsigned int dat)
{
	dt10_tp	tp;

	tp.addr		= addr | 0x08;	/* EventTrigger */
	tp.data		= ((unsigned short)(dat & 0x0FFF)) << 4;
	tp.size		= 0;
	tp.fast_mode = 0;
	gpio_write(tp);
}
EXPORT_SYMBOL(_TP_EventTrigger);

void _TP_BusOutFast( unsigned int step, unsigned int bit )
{
	dt10_tp	tp;

	tp.addr		= step << 2;
	tp.data		= bit + 2;
	tp.size		= 0;
	tp.fast_mode = 1;
	gpio_write(tp);
}
EXPORT_SYMBOL(_TP_BusOutFast);

void _TP_MemoryOutputFast( unsigned int step, unsigned int bit, void *value, unsigned int size)
{
	dt10_tp	tp;
	int	i;
	unsigned char *p;

	tp.addr		= (step << 2) + 0x01;		/* Memory Dump MASK(Fast Mode) */
	tp.data		= bit + 2;
	tp.fast_mode = 1;

	if (size >= 256) size = 256;
	tp.p[0] = size;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i++) {
			tp.p[1+i] = p[i];		/* Memory Dump */
		}
	}
	tp.size = size + 1;
	gpio_write(tp);
}
EXPORT_SYMBOL(_TP_MemoryOutputFast);

void _TP_EventTriggerFast(unsigned int dat)
{
	dt10_tp	tp;

	tp.addr		= 0x02; /* EventTrigger(Fast Mode) */
	tp.data		= dat;
	tp.size		= 0;
	tp.fast_mode = 2;
	gpio_write(tp);
}
EXPORT_SYMBOL(_TP_EventTriggerFast);
/********************************************************************************/

/*===============================================================================
	Func	: dt10drvgpio_release
	Desc	: This function is executed when closing the device driver.
===============================================================================*/
static int dt10drvgpio_release(struct inode *inode, struct file *filp)
{
	printk("dt10drvgpio: release\n");
	return 0;
}

/*===============================================================================
	Func	: file_operations driver_fops
===============================================================================*/
static struct file_operations driver_fops = {
	.write = dt10drvgpio_write,
	.open = dt10drvgpio_open,
	.release = dt10drvgpio_release,
};

/*===============================================================================
	Func	: miscdevice misc
===============================================================================*/
static struct miscdevice misc = {
	.minor = MISC_DYNAMIC_MINOR,
	.name = "dt10drvgpio",
	.fops = &driver_fops,
};

/*===============================================================================
	Func	: init_module_gpio
	Desc	: This function is executed when installing the device driver.
===============================================================================*/
int __init init_module_gpio(void)
{
	int ret;

	ret = misc_register(&misc);
	if (ret < 0) {
		printk("dt10drvgpio: misc_register() failed.\n");
		return ret;
	}

	printk("dt10drvgpio: Success init_module.\n");

	return 0;
}

/*===============================================================================
	Func	: cleanup_module_gpio
	Desc	: This function is executed when uninstalling the device driver.
===============================================================================*/
void __exit cleanup_module_gpio(void)
{
	misc_deregister(&misc);
	printk("dt10drvgpio: cleanup_module_gpio\n");
}

module_init(init_module_gpio);
module_exit(cleanup_module_gpio);

MODULE_LICENSE(LICENSE);

